/***********************************************************************************************************************
 *
 * blueBill Mobile - open source birdwatching
 * ==========================================
 *
 * Copyright (C) 2009, 2010 by Tidalwave s.a.s. (http://www.tidalwave.it)
 * http://bluebill.tidalwave.it/mobile/
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: LocationPreferencesSupport.java,v 1c2bb065fa82 2010/05/03 22:18:35 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.mobile.location;

import javax.annotation.Nonnull;
import org.openide.util.Lookup;
import it.tidalwave.geo.Coordinate;
import it.tidalwave.geo.Range;
import it.tidalwave.mobile.PreferencesAdapter;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: $
 *
 **********************************************************************************************************************/
public abstract class LocationPreferencesSupport implements LocationPreferences
  {
    protected final PreferencesAdapter preferencesAdapter = Lookup.getDefault().lookup(PreferencesAdapter.class);

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public String format (final @Nonnull Coordinate coordinate)
      {
        final double lat = coordinate.getLatitude();
        final double lon = coordinate.getLongitude();
        final String latStr = (lat >= 0) ? "N" : "S";
        final String lonStr = (lon >= 0) ? "E" : "W";
        return String.format("%.5f %s %.5f %s", Math.abs(lat), latStr, Math.abs(lon), lonStr);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public String format (final @Nonnull Range range)
      {
        return String.format("%s (\u007E%.0fm)", format(range.getCoordinate()), range.getRadius());
      }
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public boolean isReverseGeocodingEnabled()
      {
        return preferencesAdapter.getBoolean("enableReverseGeocoding", false);
      }
  }
