/***********************************************************************************************************************
 *
 * blueBill Mobile - open source birdwatching
 * ==========================================
 *
 * Copyright (C) 2009, 2010 by Tidalwave s.a.s. (http://www.tidalwave.it)
 * http://bluebill.tidalwave.it/mobile/
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: ReportGenerator.java,v f4817c97d29a 2010/05/14 22:43:52 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.mobile.observation;

import javax.annotation.Nonnull;
import java.util.Calendar;
import java.util.Date;
import org.openide.util.Lookup;
import it.tidalwave.util.AsException;
import it.tidalwave.geo.Range;
import it.tidalwave.bluebill.observation.Location;
import it.tidalwave.bluebill.observation.Observation;
import it.tidalwave.bluebill.observation.ObservationItem;
import it.tidalwave.bluebill.mobile.location.LocationPreferences;
import it.tidalwave.bluebill.mobile.GeneralPreferences;
import static it.tidalwave.bluebill.mobile.observation.ObservationPreferences.Extra.*;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: $
 *
 **********************************************************************************************************************/
public class ReportGenerator implements ObservationSetVisitor.Listener
  {
    private static final Class<Range> Range = Range.class; // FIXME: move to Range

    @Nonnull
    private final Lookup lookup = Lookup.getDefault();

    private ObservationPreferences observationPreferences;

    private final StringBuilder buffer = new StringBuilder();

    private int previousDay = 0;

    private final Calendar calendar = Calendar.getInstance();

    @Nonnull
    private final GeneralPreferences generalPreferences = lookup.lookup(GeneralPreferences.class);

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public void begin()
      {
        observationPreferences = lookup.lookup(ObservationPreferences.class);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public void end()
      {
        buffer.append("-- \n").
               append("Report created and sent by blueBill Mobile\n").
               append("http://bluebill.tidalwave.it/mobile/\n");
      }
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public void beginVisit (final @Nonnull Observation observation)
      {
        final Date date = observation.getDate();
        final Location location = observation.getLocation();
        
        calendar.setTime(date);
        final int thisDay = calendar.get(Calendar.DAY_OF_YEAR);

        if (previousDay != thisDay)
          {
            buffer.append("").append(generalPreferences.formatDateAndTime(date));
          }
        else
          {
            buffer.append("").append(generalPreferences.formatTime(date));
          }

        previousDay = thisDay;

        buffer.append(",").append(location.getDisplayName());

        try
          {
            final LocationPreferences locationPreferences = lookup.lookup(LocationPreferences.class);
            buffer.append(" - " + locationPreferences.format(location.as(Range)));
          }
        catch (AsException e)
          {
            // Range not available, never mind
          }

        buffer.append("\n");
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public void visit (final @Nonnull ObservationItem observationItem)
      {
        buffer.append("* ").append(observationPreferences.format(observationItem, CARDINALITY, GENDER));
        buffer.append("\n");
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public void endVisit (final @Nonnull Observation observation)
      {
        buffer.append("\n");
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    protected String getMessage()
      {
        return buffer.toString();
      }
  }
